@@ -316,7 +316,11 @@ void FreeTypeFont::drawGlyphToBuffer(argb_t *buffer, uint buf_width) const
 {
     FT_Bitmap *glyph_bitmap = &d_fontFace->glyph->bitmap;
 
-    for (int i = 0; i < glyph_bitmap->rows; ++i)
+    unsigned int glyph_bitmap_height =
+      static_cast<unsigned int>(glyph_bitmap->rows);
+    unsigned int glyph_bitmap_width =
+      static_cast<unsigned int>(glyph_bitmap->width);
+    for (unsigned int i = 0;  i < glyph_bitmap_height;  ++i)
     {
         uchar *src = glyph_bitmap->buffer + (i * glyph_bitmap->pitch);
         switch (glyph_bitmap->pixel_mode)
@@ -324,7 +328,7 @@ void FreeTypeFont::drawGlyphToBuffer(argb_t *buffer, uint buf_width) const
         case FT_PIXEL_MODE_GRAY:
         {
             uchar *dst = reinterpret_cast<uchar*>(buffer);
-            for (int j = 0; j < glyph_bitmap->width; ++j)
+            for (unsigned int j = 0;  j < glyph_bitmap_width;  ++j)
             {
                 // RGBA
                 *dst++ = 0xFF;
@@ -336,7 +340,7 @@ void FreeTypeFont::drawGlyphToBuffer(argb_t *buffer, uint buf_width) const
         break;
 
         case FT_PIXEL_MODE_MONO:
-            for (int j = 0; j < glyph_bitmap->width; ++j)
+            for (unsigned int j = 0;  j < glyph_bitmap_width;  ++j)
                 buffer [j] = (src [j / 8] & (0x80 >> (j & 7))) ? 0xFFFFFFFF : 0x00000000;
             break;
 
