@@ -107,7 +107,7 @@ static void blitRGBAToD3DCOLORSurface(const uint32* src, uint32* dst,
         {
             const uint32 pixel = src[j];
             const uint32 tmp = pixel & 0x00FF00FF;
-            dst[j] = pixel & 0xFF00FF00 | (tmp << 16) | (tmp >> 16);
+            dst[j] = (pixel & 0xFF00FF00) | (tmp << 16) | (tmp >> 16);
         }
 
         dst += dest_pitch / sizeof(uint32);
@@ -127,7 +127,7 @@ static void blitD3DCOLORSurfaceToRGBA(const uint32* src, uint32* dst,
         {
             const uint32 pixel = src[j];
             const uint32 tmp = pixel & 0x00FF00FF;
-            dst[j] = pixel & 0xFF00FF00 | (tmp << 16) | (tmp >> 16);
+            dst[j] = (pixel & 0xFF00FF00) | (tmp << 16) | (tmp >> 16);
         }
 
         src += source_pitch / sizeof(uint32);
