@@ -42,8 +42,11 @@ namespace CEGUI
 //----------------------------------------------------------------------------//
 const String FalagardEditbox::TypeName("Core/Editbox");
 
-const String FalagardEditbox::UnselectedTextColourPropertyName("NormalTextColour");
-const String FalagardEditbox::SelectedTextColourPropertyName("SelectedTextColour");
+const String FalagardEditbox::UnselectedTextColourPropertyName( "NormalTextColour" );
+const String FalagardEditbox::SelectedTextColourPropertyName( "SelectedTextColour" );
+const String FalagardEditbox::ActiveSelectionColourPropertyName( "ActiveSelectionColour" );
+const String FalagardEditbox::InactiveSelectionColourPropertyName( "InactiveSelectionColour" );
+
 const float FalagardEditbox::DefaultCaretBlinkTimeout(0.66f);
 
 //----------------------------------------------------------------------------//
@@ -184,6 +187,8 @@ size_t FalagardEditbox::getCaretIndex(const String& visual_text) const
         if (!firstCharRtl)
             caretIndex--;
     }
+#else
+    CEGUI_UNUSED(visual_text);
 #endif
 
     return caretIndex;
@@ -376,6 +381,11 @@ void FalagardEditbox::renderTextBidi(const WidgetLookFeel& wlf,
 
         }
     }
+#else
+    CEGUI_UNUSED(wlf);
+    CEGUI_UNUSED(text);
+    CEGUI_UNUSED(text_area);
+    CEGUI_UNUSED(text_offset);
 #endif
 }
 
