@@ -210,7 +210,7 @@ bool WidgetDemo::handleSkinSelectionAccepted(const CEGUI::EventArgs& args)
     return true;
 }
 
-bool WidgetDemo::handleRenderingEnded(const CEGUI::EventArgs& args)
+bool WidgetDemo::handleRenderingEnded(const CEGUI::EventArgs&)
 {
     d_windowLightMouseMoveEvent->disable();
     d_windowLightUpdatedEvent->disable();
@@ -237,10 +237,8 @@ bool WidgetDemo::handleRootWindowUpdate(const CEGUI::EventArgs& args)
     return true;
 }
 
-bool WidgetDemo::handleWidgetSelectionChanged(const CEGUI::EventArgs& args)
+bool WidgetDemo::handleWidgetSelectionChanged(const CEGUI::EventArgs&)
 {
-    const WindowEventArgs& winArgs = static_cast<const CEGUI::WindowEventArgs&>(args);
-
     //Get widget name and mapping type
     CEGUI::String widgetName;
     CEGUI::String widgetTypeString;
@@ -558,9 +556,9 @@ void WidgetDemo::logFiredEvent(const CEGUI::String& logMessage)
     eventsLog = eventsLog.substr(pos, len);
     if(len == 2056)
     {
-        int newlinePos = eventsLog.find_first_of("\n");
-        if(newlinePos != std::string::npos)
-            eventsLog = eventsLog.substr(newlinePos, std::string::npos);
+        String::size_type newlinePos = eventsLog.find_first_of("\n");
+        if(newlinePos != String::npos)
+            eventsLog = eventsLog.substr(newlinePos, String::npos);
     }
     d_widgetsEventsLog->setText(eventsLog);
 
@@ -951,13 +949,13 @@ void WidgetDemo::fillWidgetPropertiesDisplayWindow(CEGUI::Window* widgetWindowRo
         // Set the third row item (type) for the property
         d_widgetPropertiesDisplayWindow->setItem(new MyListItem(curProperty->getDataType()), 1, rowID);
 
-        try
+        CEGUI_TRY
         {
             // Set the second row item (value) for the property if it is gettable
             if(widget->isPropertyPresent(curProperty->getName()))
                 d_widgetPropertiesDisplayWindow->setItem(new MyListItem(widget->getProperty(curProperty->getName())), 2, rowID);
         }
-        catch(CEGUI::InvalidRequestException exception)
+        CEGUI_CATCH(CEGUI::InvalidRequestException exception)
         {
         }
 
@@ -1040,4 +1038,4 @@ extern "C" SAMPLE_EXPORT Sample& getSampleInstance()
 {
     static WidgetDemo sample;
     return sample;
-}
\ No newline at end of file
+}
