@@ -34,7 +34,7 @@
 
 // this controls conditional compile of file for Apple
 #include "CEGUISamplesConfig.h"
-#ifdef CEGUI_BUILD_RENDERER_OGRE
+#ifdef CEGUI_SAMPLES_RENDERER_OGRE_ACTIVE
 
 #include <OgreWindowEventUtilities.h>
 #include "CEGuiOgreBaseApplication.h"
@@ -56,8 +56,13 @@ CEGuiOgreBaseApplication::CEGuiOgreBaseApplication() :
 #else
     Ogre::String pluginsFileName = "plugins.cfg";
 #endif
-
-    d_ogreRoot = new Root(pluginsFileName);
+#ifndef OGRE_STATIC_LIB
+    d_ogreRoot = new Ogre::Root(pluginsFileName);
+#else
+    d_ogreRoot = new Ogre::Root();
+    d_staticPluginLoader = new Ogre::StaticPluginLoader();
+    d_staticPluginLoader->load();
+#endif
 
     setupDefaultConfigIfNeeded();
 
@@ -113,6 +118,41 @@ CEGuiOgreBaseApplication::CEGuiOgreBaseApplication() :
 
 //----------------------------------------------------------------------------//
 CEGuiOgreBaseApplication::~CEGuiOgreBaseApplication()
+{
+#ifdef OGRE_STATIC_LIB
+    d_staticPluginLoader->unload();
+    delete d_staticPluginLoader;
+#endif
+    delete d_ogreRoot;
+    delete d_windowEventListener;
+}
+
+//----------------------------------------------------------------------------//
+bool CEGuiOgreBaseApplication::init(SamplesFrameworkBase* sampleApp,
+  const CEGUI::String &logFile, const CEGUI::String &dataPathPrefixOverride)
+{
+    if (!CEGuiBaseApplication::init(sampleApp, logFile, dataPathPrefixOverride))
+        return false;
+
+    // if base initialisation failed or app was cancelled by user, bail out now.
+    if (!d_ogreRoot || !d_initialised)
+        return false;
+
+    Ogre::ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
+
+    // start rendering via Ogre3D engine.
+    CEGUI_TRY
+    {
+        d_ogreRoot->startRendering();
+    }
+    CEGUI_CATCH(...)
+    {}
+
+    return true;
+}
+
+//----------------------------------------------------------------------------//
+void CEGuiOgreBaseApplication::destroyRenderer()
 {
     delete d_frameListener;
 
@@ -123,28 +163,6 @@ CEGuiOgreBaseApplication::~CEGuiOgreBaseApplication()
     renderer.destroyOgreImageCodec(
         *static_cast<CEGUI::OgreImageCodec*>(d_imageCodec));
     CEGUI::OgreRenderer::destroy(renderer);
-
-    delete d_ogreRoot;
-    delete d_windowEventListener;
-}
-
-//----------------------------------------------------------------------------//
-void CEGuiOgreBaseApplication::run()
-{
-    // if base initialisation failed or app was cancelled by user, bail out now.
-    if (!d_ogreRoot || !d_initialised)
-        return;
-
-    Ogre::ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
-    d_sampleApp->initialise();
-
-    // start rendering via Ogre3D engine.
-    CEGUI_TRY
-    {
-        d_ogreRoot->startRendering();
-    }
-    CEGUI_CATCH(...)
-    {}
 }
 
 //----------------------------------------------------------------------------//
@@ -154,7 +172,7 @@ void CEGuiOgreBaseApplication::destroyWindow()
 }
 
 //----------------------------------------------------------------------------//
-void CEGuiOgreBaseApplication::beginRendering(const float elapsed)
+void CEGuiOgreBaseApplication::beginRendering(const float /*elapsed*/)
 {
     // this is nover called under Ogre, since we're not in control of the
     // rendering process.
@@ -188,34 +206,46 @@ void CEGuiOgreBaseApplication::initialiseResourceGroupDirectories()
     // add CEGUI sample framework datafile dirs as resource locations
     ResourceGroupManager::getSingleton().addResourceLocation("./", "FileSystem");
 
-    const char* dataPathPrefix = getDataPathPrefix();
-    char resourcePath[PATH_MAX];
+    CEGUI::String dataPathPrefix(getDataPathPrefix());
         
-    // for each resource type, set a resource group directory
-    sprintf(resourcePath, "%s/%s", dataPathPrefix, "schemes/");
-    ResourceGroupManager::getSingleton().addResourceLocation(resourcePath, "FileSystem", "schemes");
-    sprintf(resourcePath, "%s/%s", dataPathPrefix, "samples/");
-    ResourceGroupManager::getSingleton().addResourceLocation(resourcePath, "FileSystem", "samples");
-    sprintf(resourcePath, "%s/%s", dataPathPrefix, "imagesets/");
-    ResourceGroupManager::getSingleton().addResourceLocation(resourcePath, "FileSystem", "imagesets");
-    sprintf(resourcePath, "%s/%s", dataPathPrefix, "fonts/");
-    ResourceGroupManager::getSingleton().addResourceLocation(resourcePath, "FileSystem", "fonts");
-    sprintf(resourcePath, "%s/%s", dataPathPrefix, "layouts/");
-    ResourceGroupManager::getSingleton().addResourceLocation(resourcePath, "FileSystem", "layouts");
-    sprintf(resourcePath, "%s/%s", dataPathPrefix, "looknfeel/");
-    ResourceGroupManager::getSingleton().addResourceLocation(resourcePath, "FileSystem", "looknfeels");
-    sprintf(resourcePath, "%s/%s", dataPathPrefix, "lua_scripts/");
-    ResourceGroupManager::getSingleton().addResourceLocation(resourcePath, "FileSystem", "lua_scripts");
-    sprintf(resourcePath, "%s/%s", dataPathPrefix, "animations/");
-    ResourceGroupManager::getSingleton().addResourceLocation(resourcePath, "FileSystem", "animations");
-    sprintf(resourcePath, "%s/%s", dataPathPrefix, "xml_schemas/");
-    ResourceGroupManager::getSingleton().addResourceLocation(resourcePath, "FileSystem", "schemas");
+    /* for each resource type, set a resource group directory. We cast strings
+       to "const CEGUI::utf8*" in order to support general Unicode strings,
+       rather than only ASCII strings (even though currently they're all ASCII).
+       */
+    ResourceGroupManager::getSingleton().addResourceLocation((dataPathPrefix
+      +reinterpret_cast<const CEGUI::utf8*>("/schemes/")).c_str(),
+      "FileSystem", "schemes");
+    ResourceGroupManager::getSingleton().addResourceLocation((dataPathPrefix
+      +reinterpret_cast<const CEGUI::utf8*>("/samples/")).c_str(),
+      "FileSystem", "samples");
+    ResourceGroupManager::getSingleton().addResourceLocation((dataPathPrefix
+      +reinterpret_cast<const CEGUI::utf8*>("/imagesets/")).c_str(),
+      "FileSystem", "imagesets");
+    ResourceGroupManager::getSingleton().addResourceLocation((dataPathPrefix
+      +reinterpret_cast<const CEGUI::utf8*>("/fonts/")).c_str(),
+      "FileSystem", "fonts");
+    ResourceGroupManager::getSingleton().addResourceLocation((dataPathPrefix
+      +reinterpret_cast<const CEGUI::utf8*>("/layouts/")).c_str(),
+      "FileSystem", "layouts");
+    ResourceGroupManager::getSingleton().addResourceLocation((dataPathPrefix
+      +reinterpret_cast<const CEGUI::utf8*>("/looknfeel/")).c_str(),
+      "FileSystem", "looknfeels");
+    ResourceGroupManager::getSingleton().addResourceLocation((dataPathPrefix
+      +reinterpret_cast<const CEGUI::utf8*>("/lua_scripts/")).c_str(),
+      "FileSystem", "lua_scripts");
+    ResourceGroupManager::getSingleton().addResourceLocation((dataPathPrefix
+      +reinterpret_cast<const CEGUI::utf8*>("/animations/")).c_str(),
+      "FileSystem", "animations");
+    ResourceGroupManager::getSingleton().addResourceLocation((dataPathPrefix
+      +reinterpret_cast<const CEGUI::utf8*>("/xml_schemas/")).c_str(),
+      "FileSystem", "schemas");
 }
 
 //----------------------------------------------------------------------------//
-void CEGuiOgreBaseApplication::doFrameUpdate(const float elapsed)
+void CEGuiOgreBaseApplication::doFrameUpdate(const float /*elapsed*/)
 {
 }
+
 //----------------------------------------------------------------------------//
 bool CEGuiOgreBaseApplication::frameRenderingQueued(const Ogre::FrameEvent& args)
 {
@@ -295,7 +325,7 @@ void CEGuiOgreBaseApplication::setupDefaultConfigIfNeeded()
 
 //----------------------------------------------------------------------------//
 CEGuiDemoFrameListener::CEGuiDemoFrameListener(CEGuiOgreBaseApplication* baseApp, SamplesFrameworkBase*& sampleApp,
-    Ogre::RenderWindow* window, Ogre::Camera* camera, bool useBufferedInputKeys, bool useBufferedInputMouse)
+    Ogre::RenderWindow* window, Ogre::Camera* camera)
     : d_baseApp(baseApp),
     d_sampleApp(sampleApp)
 {
@@ -387,7 +417,7 @@ bool CEGuiDemoFrameListener::frameStarted(const Ogre::FrameEvent& evt)
 }
 
 //----------------------------------------------------------------------------//
-bool CEGuiDemoFrameListener::frameEnded(const Ogre::FrameEvent& evt)
+bool CEGuiDemoFrameListener::frameEnded(const Ogre::FrameEvent&)
 {
     return true;
 }
@@ -412,7 +442,7 @@ bool CEGuiDemoFrameListener::keyPressed(const OIS::KeyEvent &e)
 
 
 //----------------------------------------------------------------------------//
-bool CEGuiDemoFrameListener::keyReleased(const OIS::KeyEvent &e)
+bool CEGuiDemoFrameListener::keyReleased(const OIS::KeyEvent& e)
 {
     d_sampleApp->injectKeyUp(static_cast<CEGUI::Key::Scan>(e.key));
 
@@ -420,18 +450,18 @@ bool CEGuiDemoFrameListener::keyReleased(const OIS::KeyEvent &e)
 }
 
 //----------------------------------------------------------------------------//
-bool CEGuiDemoFrameListener::mousePressed(const OIS::MouseEvent &e, OIS::MouseButtonID id)
+bool CEGuiDemoFrameListener::mousePressed
+  (const OIS::MouseEvent&, OIS::MouseButtonID id)
 {
     d_sampleApp->injectMouseButtonDown(convertOISButtonToCegui(id));
-
     return true;
 }
 
 //----------------------------------------------------------------------------//
-bool CEGuiDemoFrameListener::mouseReleased(const OIS::MouseEvent &e, OIS::MouseButtonID id)
+bool CEGuiDemoFrameListener::mouseReleased
+  (const OIS::MouseEvent&, OIS::MouseButtonID id)
 {
-   d_sampleApp->injectMouseButtonUp(convertOISButtonToCegui(id));
-
+    d_sampleApp->injectMouseButtonUp(convertOISButtonToCegui(id));
     return true;
 }
 
