@@ -39,14 +39,13 @@ author:     Lukas E Meindl
 #include "CEGUI/widgets/PushButton.h"
 #include "CEGUI/widgets/ProgressBar.h"
 
-
 #include <string>
 #include <iostream>
 
 using namespace CEGUI;
 
 //platform-dependant DLL delay-loading includes
-#if (defined( __WIN32__ ) || defined( _WIN32 )) 
+#if (defined( __WIN32__ ) || defined( _WIN32 ))
 #include "windows.h"
 #endif
 
@@ -54,24 +53,42 @@ using namespace CEGUI;
 // Name of the xsd schema file used to validate samples XML files.
 const String SamplesFramework::XMLSchemaName("Samples.xsd");
 
+#if !defined __ANDROID__
+
+/* Usage:
+   <sample framework executable> [-d <path to data files>] [<XML file name>]
+   */
+
 //----------------------------------------------------------------------------//
-int main(int argc, char* argv[])
+int main(int /*argc*/, char* argv[])
 {
-    // Basic start-up for the sample browser application.
-    // Will remain in run() until quitting
-    int argidx = 1;
-#ifdef __APPLE__
-    if (argc > 1 && !std::strncmp(argv[argidx], "-psn", 4))
+    String xml_filename;
+    String data_path_prefix_override;
+    char** arg = argv;
+    while (true)
     {
-        --argc;
-        ++argidx;
-    }
-#endif
-    
-    SamplesFramework sampleFramework(argc > 1 ? argv[argidx] : "");
-    return sampleFramework.run();
+        ++arg;
+        if (!*arg)
+            break;
+        if ((*arg)[0] == '-')
+        {
+            if ((*arg)[1] == 'd')
+            {
+                ++arg;
+                if (!*arg)
+                    break;
+                data_path_prefix_override = *arg;
+            }
+        }
+        else
+           xml_filename = *arg;
+   }
+    SamplesFramework sampleFramework(xml_filename);
+    return sampleFramework.run(data_path_prefix_override);
 }
 
+#endif // !defined __ANDROID__
+
 //----------------------------------------------------------------------------//
 SamplesFramework::SamplesFramework(const CEGUI::String& xml_filename) :
     d_sampleExitButton(0),
@@ -91,24 +108,23 @@ SamplesFramework::~SamplesFramework()
         delete d_metaDataWinMgr;
 }
 
-
-
 //----------------------------------------------------------------------------//
-bool SamplesFramework::initialise()
+bool SamplesFramework::initialise(const CEGUI::String& logFile,
+                                  const CEGUI::String& dataPathPrefixOverride)
 {
-    using namespace CEGUI;
-
-    initialiseLoadScreenLayout();
-
-    // return true to signalize the initialisation was sucessful and run the
-    // SamplesFramework
-    return true;
+    if (SamplesFrameworkBase::initialise(logFile, dataPathPrefixOverride))
+    {
+        initialiseLoadScreenLayout();
+        return true;
+    }
+    return false;
 }
 
 //----------------------------------------------------------------------------//
-void SamplesFramework::deinitialise()
+void SamplesFramework::cleanup()
 {
     unloadSamples();
+    SamplesFrameworkBase::cleanup();
 }
 
 //----------------------------------------------------------------------------//
@@ -441,7 +457,8 @@ SampleData* SamplesFramework::findSampleData(CEGUI::Window* sampleWindow)
 }
 
 //----------------------------------------------------------------------------//
-bool SamplesFramework::handleSampleExitButtonClicked(const CEGUI::EventArgs& args)
+bool SamplesFramework::handleSampleExitButtonClicked
+  (const CEGUI::EventArgs&)
 {
     d_quittingSampleView = true;
 
@@ -534,7 +551,7 @@ bool SamplesFramework::updateInitialisationStep()
     case 0:
     {
         const String filename(d_samplesXMLFilename.empty() ?
-            String(d_baseApp->getDataPathPrefix()) + "/samples/samples.xml" :
+            d_baseApp->getDataPathPrefix() + "/samples/samples.xml" :
             d_samplesXMLFilename);
 
         loadSamplesDataFromXML(filename);
